#!/bin/bash
#
# check_stale_fs - Nagios plugin, checks stale filesystems with df command
# by Michael Tosch
#
# optional ARG1: timeout in seconds
# The default timeout is 45 seconds, then it kills itself and returns a warning
# Run this with "nrpe -t 50" to prevent an "nrpe: connection timout"
#
# returns only OK or WARNING

set -f
PATH=/usr/bin:/bin:/usr/sbin:/sbin
export PATH

# We need 'exec' to propagate the timer
# therefore we call ourself ('fork') with "-timeout"
if [ "$1" = "-timeout" ]
then
  t=$2
  shift 2
  (
    # $$ is the pid of the calling shell
    while ((t > 0)); do
      sleep 2
      kill -0 $$ || exit 0
      ((t -= 2))
    done
    # Be nice, post SIGTERM first.
    # The 'exit 0' below will be executed if any preceding command fails.
    kill -15 $$
    sleep 2
    kill -0 $$ || exit 0
    sleep 1
    kill -9 $$
  ) >/dev/null 2>&1 </dev/null &
  exec "$@"
  # won't continue here
fi

TO=45
case $1 in
*[!0-9]*) echo "Usage: $0 [timeout]
 timeout in seconds, default $TO"; exit 3;;
?*) TO=$1; shift;;
esac

# a simple df does a statfs() on all mounted fs, but only stat() might return "stale".
# So we need to run df on each mounted fs - that does both stat() and statfs().
# For efficiency (less forks) we use xargs.

# mount command output is different between Linux and Unix
# but mtab is similar

for mtab in /etc/mnttab /etc/mtab /proc/mounts
do
  [ -r $mtab ] && break
done
if [ ! -r $mtab ]
then
  mtab=/tmp/mtab.$$
  trap "rm -f $mtab" 0 HUP INT SYS
  mount | awk '$1~/^\//' > $mtab
fi

stderr=`
awk '$3~/^(nfs|mvfs|afs)$/ {print $2}' $mtab | xargs $0 -timeout $TO df 2>&1 >/dev/null
`
errstate=$?

# these are not stale:
stderr2=`echo "$stderr" | fgrep -v "\
Permission denied"`
# 13.06.2014 removed: "not a block device"

# NB: Linux xargs changes command exit status 1 to 123
if [ $errstate -eq 0 -a -z "$stderr" ] || [ \( $errstate -eq 1 -o $errstate -eq 123 \) -a -z "$stderr2" ]
then
  echo "OK: no stale filesystems"
  exit 0
else
  echo "WARNING: stale filesystems. Please check."
  exit 1
fi

